//
//  AEAudioController.m
//  TAAESample
//
//  Created by Michael Tyson on 24/03/2016.
//  Copyright © 2016 A Tasty Pixel. All rights reserved.
//
// Strictly for educational purposes only. No part of TAAESample is to be distributed
// in any form other than as source code within the TAAE2 repository.

#import "AEAudioController.h"
@import AVFoundation;

//
//static const AESeconds kCountInThreshold = 0.2;
//static const double kMicBandpassCenterFrequency = 2000.0;

@interface AEAudioController ()
//@property (nonatomic, strong, readwrite) AEAudioUnitInputModule * input;
@property (nonatomic, strong) AEAudioUnitOutput * output;
@property (nonatomic,strong,readwrite) AEBandpassModule * bandpass;
@property (nonatomic,strong) AEManagedValue * recorderValue;
@property (nonatomic,strong) NSURL *recordingPath;//computed value

//@property (nonatomic, strong, readwrite) AEVarispeedModule * varispeed;
//@property (nonatomic, strong, readwrite) AEAudioFilePlayerModule * drums;
@property (nonatomic, strong, readwrite) AEAudioFilePlayerModule * amen;
//@property (nonatomic, strong, readwrite) AEAudioFilePlayerModule * piano;
//@property (nonatomic, strong, readwrite) AEAudioFilePlayerModule * sample1;
//@property (nonatomic, strong, readwrite) AEAudioFilePlayerModule * sample2;
//@property (nonatomic, strong, readwrite) AEAudioFilePlayerModule * sample3;
//@property (nonatomic, strong, readwrite) AEAudioFilePlayerModule * sweep;
//@property (nonatomic, strong, readwrite) AEAudioFilePlayerModule * hit;
//@property (nonatomic, strong, readwrite) AEBandpassModule * bandpass;
//@property (nonatomic, strong, readwrite) AEBandpassModule * micBandpass;
//@property (nonatomic, readwrite) BOOL recording;
//@property (nonatomic, readwrite) BOOL playingRecording;
//@property (nonatomic, strong) AEManagedValue * recorderValue;
//@property (nonatomic, strong) AEManagedValue * playerValue;
//@property (nonatomic) BOOL playingThroughSpeaker;
//@property (nonatomic, strong) id routeChangeObserverToken;
@end

@implementation AEAudioController
@dynamic recordingPath;
-(instancetype)init{
    if ( !(self = [super init]) ) return nil;
    
    
    AERenderer * renderer = [AERenderer new];
    
    self.output = [[AEAudioUnitOutput alloc] initWithRenderer:renderer];//get output sooner
    NSError * error;
    
    AEAudioFilePlayerModule * amen = [[AEAudioFilePlayerModule alloc]initWithRenderer:renderer URL:[[NSBundle mainBundle]URLForResource:@"clap" withExtension:@"wav"] error:&error];//init sample
    amen.loop = false;//loop forever
 
    [amen playAtTime:0];//lets you sync
    
    AEBandpassModule * bandpass = [[AEBandpassModule alloc]initWithRenderer:renderer];//init bandpass
    self.bandpass = bandpass;
    
    AEManagedValue * recorderValue = [AEManagedValue new];
    self.recorderValue = recorderValue;
    
    AEAudioUnitInputModule * input = self.output.inputModule;//mic input
    
    AEDelayModule * delay = [[AEDelayModule alloc]initWithRenderer:renderer];
    delay.delayTime = amen.duration / 16.0; //amen loop length divided by 16
    
    renderer.block = ^(const AERenderContext * context){//top level of audio processing
        AEModuleProcess(amen,context);
       // AEModuleProcess(bandpass,context);
        
        
        //AEModuleProcess(input, context);
        //AEModuleProcess(delay, context);
        
        //mix two buffers amen break with bandpass + input mix with delay = ->output
        AEBufferStackMix(context->stack,2);
        
        __unsafe_unretained AEFileRecorderModule * recorder = (__bridge AEFileRecorderModule*)AEManagedValueGetValue(recorderValue);
        if(recorder){
            AEModuleProcess(recorder, context);
        }
        
        //separate chain input mic
        
        
        AEBufferStackMixToBufferList(context->stack,1,0,YES,context->output);
    };
    
    
    return self;
}





-(BOOL)start:(NSError * _Nullable * _Nullable)error{
    [[AVAudioSession sharedInstance] setCategory:AVAudioSessionCategoryPlayAndRecord
                                     withOptions:AVAudioSessionCategoryOptionMixWithOthers | AVAudioSessionCategoryOptionDefaultToSpeaker
                                           error:NULL];
    return [self.output start:error];
   
}

-(void)stop{
    [self.output stop];
}

-(void)setRecording:(BOOL)recording{
    if(_recording==recording) return;
    
    _recording = recording;
    
    if(_recording){
        AEFileRecorderModule * recorder = [[AEFileRecorderModule alloc]initWithRenderer:self.output.renderer
                                                                                    URL:self.recordingPath
                                                                                   type:kAudioFileWAVEType
                                                                                  error:NULL];
        [recorder beginRecordingAtTime:0];
        self.recorderValue.objectValue = recorder;//change the value and retain it
        
    }else{
        
        [(AEFileRecorderModule*)self.recorderValue.objectValue stopRecordingAtTime:0 completionBlock:^{
            self.recorderValue.objectValue = nil;
        }];
        
    }
}

-(NSURL *)recordingPath{
             return [[[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask].firstObject URLByAppendingPathComponent:@"Recording2.m4a"];
    
    
    
}

@end
